/*******************************************************************************
 * Copyright (c) 2013 Max Göbel.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser Public License v2.1
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 * 
 * Contributors:
 *     Max Göbel - initial API and implementation
 ******************************************************************************/
package at.tuwien.prip.model.attributes.html;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;


import at.tuwien.prip.common.log.ErrorDump;
import at.tuwien.prip.common.utils.ListUtils;
import at.tuwien.prip.model.attributes.domains.lxEnumValueDef;

/**
 * HTMLAttributes.java
 *
 *
 * Created: Sun Jul 27 18:56:41 2003
 *
 * @author Michal Ceresna
 * @version 1.0
 */
public abstract class HTMLAttributes
{

	public static HTMLAttributeDef getDefinitionByName(String tag_name,
			String att_name)
	{
		List<HTMLAttributeDef> defs = name2defs.get(att_name);
		if (defs!=null) {
			for (HTMLAttributeDef def : defs) {
				HTMLTagDefSet tags = def.getTags();
				if (tags.contains(tag_name)) {
					return def;
				}
			}
			//well, this probably violates the html standard,
			//so take the first att. definition regardless
			//of allowed tags-context
			if (!defs.isEmpty()) {
				return defs.get(0);
			}
		}

		//unknown attribute
		ErrorDump.error(HTMLAttributes.class,
				"log_unknown_attribute"+
				tag_name+
				att_name
		);
		return
		new HTMLAttributeDef(att_name,
				HTMLAttributeDef.RENDERING_EFFECT_BY_PRESENCE,
				HTMLValueDomains.D_CDATA,
				HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
				new Comment("unknown attribute"),
				new HTMLNegTagDefSet(new HTMLTagDef[] {}));
	}

	private static final HashMap<String, List<HTMLAttributeDef>> name2defs;

	// build a list of all allowed html attrribute definitions
	static {
		
		HTMLAttributeDef[] attdefs = new HTMLAttributeDef[] {
				new HTMLAttributeDef("abbr",
						HTMLAttributeDef.RENDERING_EFFECT_NONE,
						HTMLValueDomains.D_TEXT,
						HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
						new Comment("abbreviation for header cell"),
						new HTMLEnumTagDefSet(new HTMLTagDef[] {
								HTMLTags.T_TD, HTMLTags.T_TH
						})),

						new HTMLAttributeDef("accept-charset",
								HTMLAttributeDef.RENDERING_EFFECT_NONE,
								HTMLValueDomains.D_CHARSETS,
								HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
								new Comment("list of supported charsets"),
								new HTMLEnumTagDefSet(new HTMLTagDef[] {
										HTMLTags.T_FORM
								})),

								new HTMLAttributeDef("accept",
										HTMLAttributeDef.RENDERING_EFFECT_NONE,
										HTMLValueDomains.D_CONTENTTYPES,
										HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
										new Comment("list of MIME types for file upload"),
										new HTMLEnumTagDefSet(new HTMLTagDef[] {
												HTMLTags.T_FORM, HTMLTags.T_INPUT
										})),

										new HTMLAttributeDef("accesskey",
												HTMLAttributeDef.RENDERING_EFFECT_NONE,
												HTMLValueDomains.D_CHARACTER,
												HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
												new Comment("accessibility key character"),
												new HTMLEnumTagDefSet(new HTMLTagDef[] {
														HTMLTags.T_A, HTMLTags.T_AREA, HTMLTags.T_BUTTON, HTMLTags.T_INPUT, HTMLTags.T_LABEL, HTMLTags.T_LEGEND, HTMLTags.T_TEXTAREA
												})),

												new HTMLAttributeDef("action",
														HTMLAttributeDef.RENDERING_EFFECT_NONE,
														HTMLValueDomains.D_URI,
														HTMLAttributeDef.DEFAULT_VALUE_REQUIRED,
														new Comment("server-side form handler"),
														new HTMLEnumTagDefSet(new HTMLTagDef[] {
																HTMLTags.T_FORM
														})),

														new HTMLAttributeDef("align",
																HTMLAttributeDef.RENDERING_EFFECT_BY_EACH_VALUE,
																HTMLValueDomains.D_CALIGN,
																HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																new Comment("relative to table"),
																new HTMLEnumTagDefSet(new HTMLTagDef[] {
																		HTMLTags.T_CAPTION
																})),
																new HTMLAttributeDef("align",
																		HTMLAttributeDef.RENDERING_EFFECT_BY_EACH_VALUE,
																		HTMLValueDomains.D_IALIGN,
																		HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																		new Comment("vertical or horizontal alignment"),
																		new HTMLEnumTagDefSet(new HTMLTagDef[] {
																				HTMLTags.T_APPLET, HTMLTags.T_IFRAME, HTMLTags.T_IMG, HTMLTags.T_INPUT, HTMLTags.T_OBJECT
																		})),
																		new HTMLAttributeDef("align",
																				HTMLAttributeDef.RENDERING_EFFECT_BY_EACH_VALUE,
																				HTMLValueDomains.D_LALIGN,
																				HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																				new Comment("relative to fieldset"),
																				new HTMLEnumTagDefSet(new HTMLTagDef[] {
																						HTMLTags.T_LEGEND
																				})),
																				new HTMLAttributeDef("align",
																						HTMLAttributeDef.RENDERING_EFFECT_BY_EACH_VALUE,
																						HTMLValueDomains.D_TALIGN,
																						HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																						new Comment("table position relative to window"),
																						new HTMLEnumTagDefSet(new HTMLTagDef[] {
																								HTMLTags.T_TABLE
																						})),
																						new HTMLAttributeDef("align",
																								HTMLAttributeDef.RENDERING_EFFECT_BY_EACH_VALUE,
																								HTMLValueDomains.D_HALIGN,
																								HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																								new Comment(""),
																								new HTMLEnumTagDefSet(new HTMLTagDef[] {
																										HTMLTags.T_HR
																								})),
																								//dedi sa nadol
																								new HTMLAttributeDef("align",
																										HTMLAttributeDef.RENDERING_EFFECT_BY_EACH_VALUE,
																										HTMLValueDomains.D_PALIGN,
																										HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																										new Comment("align, text alignment"),
																										new HTMLEnumTagDefSet(new HTMLTagDef[] {
																												HTMLTags.T_DIV, HTMLTags.T_H1, HTMLTags.T_H2, HTMLTags.T_H3, HTMLTags.T_H4, HTMLTags.T_H5, HTMLTags.T_H6, HTMLTags.T_P
																										})),
																										new HTMLAttributeDef("align",
																												HTMLAttributeDef.RENDERING_EFFECT_BY_EACH_VALUE,
																												HTMLValueDomains.D_AALIGN,
																												HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																												new Comment(""),
																												new HTMLEnumTagDefSet(new HTMLTagDef[] {
																														HTMLTags.T_COL, HTMLTags.T_COLGROUP, HTMLTags.T_TBODY, HTMLTags.T_TD, HTMLTags.T_TFOOT, HTMLTags.T_TH, HTMLTags.T_THEAD, HTMLTags.T_TR
																												})),

																												new HTMLAttributeDef("alink",
																														HTMLAttributeDef.RENDERING_EFFECT_NONE,
																														HTMLValueDomains.D_COLOR,
																														HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																														new Comment("color of selected links"),
																														new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																HTMLTags.T_BODY
																														})),

																														new HTMLAttributeDef("alt",
																																HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																HTMLValueDomains.D_TEXT,
																																HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																new Comment("short description"),
																																new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																		HTMLTags.T_APPLET
																																})),
																																new HTMLAttributeDef("alt",
																																		HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																		HTMLValueDomains.D_TEXT,
																																		HTMLAttributeDef.DEFAULT_VALUE_REQUIRED,
																																		new Comment("short description"),
																																		new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																				HTMLTags.T_AREA, HTMLTags.T_IMG
																																		})),
																																		new HTMLAttributeDef("alt",
																																				HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																				HTMLValueDomains.D_CDATA,
																																				HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																				new Comment("short description"),
																																				new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																						HTMLTags.T_INPUT
																																				})),

																																				new HTMLAttributeDef("archive",
																																						HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																						HTMLValueDomains.D_CDATA,
																																						HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																						new Comment("comma-separated archive list"),
																																						new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																								HTMLTags.T_APPLET
																																						})),
																																						new HTMLAttributeDef("archive",
																																								HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																								HTMLValueDomains.D_CDATA,
																																								HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																								new Comment("space-separated list of URIs"),
																																								new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																										HTMLTags.T_OBJECT
																																								})),

																																								new HTMLAttributeDef("axis",
																																										HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																										HTMLValueDomains.D_CDATA,
																																										HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																										new Comment("comma-separated list of related headers"),
																																										new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																												HTMLTags.T_TD, HTMLTags.T_TH
																																										})),

																																										new HTMLAttributeDef("background",
																																												HTMLAttributeDef.RENDERING_EFFECT_BY_EACH_VALUE,
																																												HTMLValueDomains.D_URI,
																																												HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																												new Comment("texture tile for document background"),
																																												new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																														HTMLTags.T_BODY,
																																														//html-standard violations
																																														HTMLTags.T_TD
																																												})),

																																												new HTMLAttributeDef("bgcolor",
																																														HTMLAttributeDef.RENDERING_EFFECT_BY_EACH_VALUE,
																																														HTMLValueDomains.D_COLOR,
																																														HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																														new Comment("background color for cells"),
																																														new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																HTMLTags.T_TABLE
																																														})),
																																														new HTMLAttributeDef("bgcolor",
																																																HTMLAttributeDef.RENDERING_EFFECT_BY_EACH_VALUE,
																																																HTMLValueDomains.D_COLOR,
																																																HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																new Comment("background color for row"),
																																																new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																		HTMLTags.T_TR
																																																})),
																																																new HTMLAttributeDef("bgcolor",
																																																		HTMLAttributeDef.RENDERING_EFFECT_BY_EACH_VALUE,
																																																		HTMLValueDomains.D_COLOR,
																																																		HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																		new Comment("cell background color"),
																																																		new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																				HTMLTags.T_TD, HTMLTags.T_TH
																																																		})),
																																																		new HTMLAttributeDef("bgcolor",
																																																				HTMLAttributeDef.RENDERING_EFFECT_BY_EACH_VALUE,
																																																				HTMLValueDomains.D_COLOR,
																																																				HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																				new Comment("document background color"),
																																																				new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																						HTMLTags.T_BODY
																																																				})),

																																																				new HTMLAttributeDef("border",
																																																						HTMLAttributeDef.RENDERING_EFFECT_BY_EACH_VALUE,
																																																						HTMLValueDomains.D_PIXELS,
																																																						HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																						new Comment("controls frame width around table"),
																																																						new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																								HTMLTags.T_TABLE
																																																						})),
																																																						new HTMLAttributeDef("border",
																																																								HTMLAttributeDef.RENDERING_EFFECT_BY_EACH_VALUE,
																																																								HTMLValueDomains.D_PIXELS,
																																																								HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																								new Comment("link border width"),
																																																								new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																										HTMLTags.T_IMG, HTMLTags.T_OBJECT
																																																								})),

																																																								new HTMLAttributeDef("cellpadding",
																																																										HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																										HTMLValueDomains.D_LENGTH,
																																																										HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																										new Comment("spacing within cells"),
																																																										new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																												HTMLTags.T_TABLE
																																																										})),

																																																										new HTMLAttributeDef("cellspacing",
																																																												HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																												HTMLValueDomains.D_LENGTH,
																																																												HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																												new Comment("spacing between cells"),
																																																												new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																														HTMLTags.T_TABLE
																																																												})),

																																																												new HTMLAttributeDef("char",
																																																														HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																														HTMLValueDomains.D_CHARACTER,
																																																														HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																														new Comment("alignment char, e.g. char=':'"),
																																																														new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																HTMLTags.T_COL, HTMLTags.T_COLGROUP, HTMLTags.T_TBODY, HTMLTags.T_TD, HTMLTags.T_TFOOT, HTMLTags.T_TH, HTMLTags.T_THEAD, HTMLTags.T_TR
																																																														})),

																																																														new HTMLAttributeDef("charoff",
																																																																HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																HTMLValueDomains.D_LENGTH,
																																																																HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																new Comment("offset for alignment char"),
																																																																new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																		HTMLTags.T_COL, HTMLTags.T_COLGROUP, HTMLTags.T_TBODY, HTMLTags.T_TD, HTMLTags.T_TFOOT, HTMLTags.T_TH, HTMLTags.T_THEAD, HTMLTags.T_TR
																																																																})),

																																																																new HTMLAttributeDef("charset",
																																																																		HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																		HTMLValueDomains.D_CHARSET,
																																																																		HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																		new Comment("char encoding of linked resource"),
																																																																		new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																				HTMLTags.T_A, HTMLTags.T_LINK, HTMLTags.T_SCRIPT
																																																																		})),

																																																																		//nededi sa
																																																																		new HTMLAttributeDef("checked",
																																																																				HTMLAttributeDef.RENDERING_EFFECT_BY_PRESENCE,
																																																																				HTMLValueDomains.D_EMPTY,
																																																																				HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																				new Comment("for radio buttons and check boxes"),
																																																																				new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																						HTMLTags.T_INPUT
																																																																				})),

																																																																				new HTMLAttributeDef("cite",
																																																																						HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																						HTMLValueDomains.D_URI,
																																																																						HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																						new Comment("URI for source document or msg"),
																																																																						new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																								HTMLTags.T_BLOCKQUOTE, HTMLTags.T_Q
																																																																						})),
																																																																						new HTMLAttributeDef("cite",
																																																																								HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																								HTMLValueDomains.D_URI,
																																																																								HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																								new Comment("info on reason for change"),
																																																																								new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																										HTMLTags.T_DEL, HTMLTags.T_INS
																																																																								})),

																																																																								new HTMLAttributeDef("class",
																																																																										HTMLAttributeDef.RENDERING_EFFECT_BY_EACH_VALUE,
																																																																										HTMLValueDomains.D_CDATA, //FIXME
																																																																										HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																										new Comment("space-separated list of classes"),
																																																																										new HTMLNegTagDefSet(new HTMLTagDef[] {
																																																																												HTMLTags.T_BASE, HTMLTags.T_BASEFONT, HTMLTags.T_HEAD, HTMLTags.T_HTML, HTMLTags.T_META,
																																																																												HTMLTags.T_PARAM, HTMLTags.T_SCRIPT, HTMLTags.T_STYLE, HTMLTags.T_TITLE
																																																																										})),

																																																																										new HTMLAttributeDef("classid",
																																																																												HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																												HTMLValueDomains.D_URI,
																																																																												HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																												new Comment("identifies an implementation"),
																																																																												new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																														HTMLTags.T_OBJECT
																																																																												})),

																																																																												new HTMLAttributeDef("clear",
																																																																														HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																														new HTMLValueDomain(lxEnumValueDef.class,
																																																																																ListUtils.toSet(new String[] {
																																																																																		"left", "all", "right", "none"
																																																																																}),
																																																																																null),
																																																																																"none",
																																																																																new Comment("control of text flow"),
																																																																																new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																		HTMLTags.T_BR
																																																																																})),

																																																																																new HTMLAttributeDef("code",
																																																																																		HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																		HTMLValueDomains.D_CDATA, //FIXME
																																																																																		HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																		new Comment("applet class file"),
																																																																																		new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																				HTMLTags.T_APPLET
																																																																																		})),

																																																																																		new HTMLAttributeDef("codebase",
																																																																																				HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																				HTMLValueDomains.D_URI,
																																																																																				HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																				new Comment("base URI for classid, data, archive"),
																																																																																				new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																						HTMLTags.T_OBJECT
																																																																																				})),
																																																																																				new HTMLAttributeDef("codebase",
																																																																																						HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																						HTMLValueDomains.D_URI,
																																																																																						HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																						new Comment("optional base URI for applet"),
																																																																																						new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																								HTMLTags.T_APPLET
																																																																																						})),

																																																																																						new HTMLAttributeDef("codetype",
																																																																																								HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																								HTMLValueDomains.D_CONTENTTYPE,
																																																																																								HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																								new Comment("content type for code"),
																																																																																								new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																										HTMLTags.T_OBJECT
																																																																																								})),

																																																																																								new HTMLAttributeDef("color",
																																																																																										HTMLAttributeDef.RENDERING_EFFECT_BY_EACH_VALUE,
																																																																																										HTMLValueDomains.D_COLOR,
																																																																																										HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																										new Comment("text color"),
																																																																																										new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																												HTMLTags.T_BASEFONT, HTMLTags.T_FONT
																																																																																										})),

																																																																																										new HTMLAttributeDef("cols",
																																																																																												HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																												HTMLValueDomains.D_MULTILENGTHS,
																																																																																												HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																												new Comment("list of lengths, HTMLTags.T_default: 100% (1 col)"),
																																																																																												new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																														HTMLTags.T_FRAMESET
																																																																																												})),
																																																																																												new HTMLAttributeDef("cols",
																																																																																														HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																														HTMLValueDomains.D_NUMBER,
																																																																																														HTMLAttributeDef.DEFAULT_VALUE_REQUIRED,
																																																																																														new Comment(""),
																																																																																														new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																HTMLTags.T_TEXTAREA,
																																																																																																//html-standard violations
																																																																																																HTMLTags.T_TABLE
																																																																																														})),

																																																																																														new HTMLAttributeDef("colspan",
																																																																																																HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																HTMLValueDomains.D_NUMBER,
																																																																																																"1",
																																																																																																new Comment("number of cols spanned by cell"),
																																																																																																new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																		HTMLTags.T_TD, HTMLTags.T_TH
																																																																																																})),

																																																																																																new HTMLAttributeDef("compact",
																																																																																																		HTMLAttributeDef.RENDERING_EFFECT_BY_PRESENCE,
																																																																																																		HTMLValueDomains.D_EMPTY,
																																																																																																		HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																		new Comment("reduced interitem spacing"),
																																																																																																		new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																				HTMLTags.T_DIR, HTMLTags.T_DL, HTMLTags.T_MENU, HTMLTags.T_OL, HTMLTags.T_UL
																																																																																																		})),

																																																																																																		new HTMLAttributeDef("content",
																																																																																																				HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																				HTMLValueDomains.D_CDATA,
																																																																																																				HTMLAttributeDef.DEFAULT_VALUE_REQUIRED,
																																																																																																				new Comment("associated information"),
																																																																																																				new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																						HTMLTags.T_META
																																																																																																				})),

																																																																																																				new HTMLAttributeDef("coords",
																																																																																																						HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																						HTMLValueDomains.D_COORDS,
																																																																																																						HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																						new Comment("comma-separated list of lengths"),
																																																																																																						new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																								HTMLTags.T_AREA
																																																																																																						})),
																																																																																																						new HTMLAttributeDef("coords",
																																																																																																								HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																								HTMLValueDomains.D_COORDS,
																																																																																																								HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																								new Comment("for use with client-side image maps"),
																																																																																																								new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																										HTMLTags.T_A
																																																																																																								})),

																																																																																																								new HTMLAttributeDef("data",
																																																																																																										HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																										HTMLValueDomains.D_URI,
																																																																																																										HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																										new Comment("reference to object's data"),
																																																																																																										new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																												HTMLTags.T_OBJECT
																																																																																																										})),

																																																																																																										new HTMLAttributeDef("datetime",
																																																																																																												HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																												HTMLValueDomains.D_DATETIME,
																																																																																																												HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																												new Comment("date and time of change"),
																																																																																																												new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																														HTMLTags.T_DEL, HTMLTags.T_INS
																																																																																																												})),

																																																																																																												new HTMLAttributeDef("declare",
																																																																																																														HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																														HTMLValueDomains.D_EMPTY,
																																																																																																														HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																														new Comment("declare but don't instantiate flag"),
																																																																																																														new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																HTMLTags.T_OBJECT
																																																																																																														})),

																																																																																																														new HTMLAttributeDef("defer",
																																																																																																																HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																HTMLValueDomains.D_EMPTY,
																																																																																																																HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																new Comment("UA may defer execution of script"),
																																																																																																																new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																		HTMLTags.T_SCRIPT
																																																																																																																})),

																																																																																																																new HTMLAttributeDef("dir",
																																																																																																																		HTMLAttributeDef.RENDERING_EFFECT_BY_EACH_VALUE,
																																																																																																																		new HTMLValueDomain(lxEnumValueDef.class,
																																																																																																																				ListUtils.toSet(new String[] {
																																																																																																																						"ltr", "rtl"
																																																																																																																				}),
																																																																																																																				null),
																																																																																																																				HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																				new Comment("direction for weak/neutral text"),
																																																																																																																				new HTMLNegTagDefSet(new HTMLTagDef[] {
																																																																																																																						HTMLTags.T_APPLET, HTMLTags.T_BASE, HTMLTags.T_BASEFONT, HTMLTags.T_BDO,
																																																																																																																						HTMLTags.T_BR, HTMLTags.T_FRAME, HTMLTags.T_FRAMESET, HTMLTags.T_IFRAME,
																																																																																																																						HTMLTags.T_PARAM, HTMLTags.T_SCRIPT
																																																																																																																				})),
																																																																																																																				new HTMLAttributeDef("dir",
																																																																																																																						HTMLAttributeDef.RENDERING_EFFECT_BY_EACH_VALUE,
																																																																																																																						new HTMLValueDomain(lxEnumValueDef.class,
																																																																																																																								ListUtils.toSet(new String[] {
																																																																																																																										"ltr", "rtl"
																																																																																																																								}),
																																																																																																																								null),
																																																																																																																								HTMLAttributeDef.DEFAULT_VALUE_REQUIRED,
																																																																																																																								new Comment("directionality"),
																																																																																																																								new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																										HTMLTags.T_BDO
																																																																																																																								})),

																																																																																																																								//nededi sa
																																																																																																																								new HTMLAttributeDef("disabled",
																																																																																																																										HTMLAttributeDef.RENDERING_EFFECT_BY_PRESENCE,
																																																																																																																										HTMLValueDomains.D_EMPTY,
																																																																																																																										HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																										new Comment("unavailable in this context"),
																																																																																																																										new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																												HTMLTags.T_BUTTON, HTMLTags.T_INPUT, HTMLTags.T_OPTGROUP, HTMLTags.T_OPTION, HTMLTags.T_SELECT, HTMLTags.T_TEXTAREA
																																																																																																																										})),

																																																																																																																										new HTMLAttributeDef("enctype",
																																																																																																																												HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																												HTMLValueDomains.D_CONTENTTYPE,
																																																																																																																												"application/x-www- form-urlencoded",
																																																																																																																												new Comment(""),
																																																																																																																												new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																														HTMLTags.T_FORM
																																																																																																																												})),

																																																																																																																												new HTMLAttributeDef("face",
																																																																																																																														HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																														HTMLValueDomains.D_CDATA,
																																																																																																																														HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																														new Comment("comma-separated list of font names"),
																																																																																																																														new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																HTMLTags.T_BASEFONT, HTMLTags.T_FONT,
																																																																																																																																//swing special
																																																																																																																																HTMLTags.T_CONTENT, HTMLTags.T_IMG
																																																																																																																														})),

																																																																																																																														new HTMLAttributeDef("for",
																																																																																																																																HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																HTMLValueDomains.D_IDREF,
																																																																																																																																HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																new Comment("matches field ID value"),
																																																																																																																																new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																		HTMLTags.T_LABEL
																																																																																																																																})),

																																																																																																																																new HTMLAttributeDef("frame",
																																																																																																																																		HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																		HTMLValueDomains.D_TFRAME,
																																																																																																																																		HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																		new Comment("which parts of frame to render"),
																																																																																																																																		new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																				HTMLTags.T_TABLE
																																																																																																																																		})),

																																																																																																																																		new HTMLAttributeDef("frameborder",
																																																																																																																																				HTMLAttributeDef.RENDERING_EFFECT_BY_EACH_VALUE,
																																																																																																																																				new HTMLValueDomain(lxEnumValueDef.class,
																																																																																																																																						ListUtils.toSet(new String[] {
																																																																																																																																								"1", "0"
																																																																																																																																						}),
																																																																																																																																						null),
																																																																																																																																						"1",
																																																																																																																																						new Comment("request frame borders?"),
																																																																																																																																						new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																								HTMLTags.T_FRAME, HTMLTags.T_IFRAME
																																																																																																																																						})),

																																																																																																																																						new HTMLAttributeDef("headers",
																																																																																																																																								HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																								HTMLValueDomains.D_IDREFS,
																																																																																																																																								HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																								new Comment("list of id's for header cells"),
																																																																																																																																								new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																										HTMLTags.T_TD, HTMLTags.T_TH
																																																																																																																																								})),

																																																																																																																																								new HTMLAttributeDef("height",
																																																																																																																																										HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																										HTMLValueDomains.D_LENGTH,
																																																																																																																																										HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																										new Comment("frame height"),
																																																																																																																																										new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																												HTMLTags.T_IFRAME
																																																																																																																																										})),
																																																																																																																																										new HTMLAttributeDef("height",
																																																																																																																																												HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																												HTMLValueDomains.D_LENGTH,
																																																																																																																																												HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																												new Comment("height for cell"),
																																																																																																																																												new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																														HTMLTags.T_TD, HTMLTags.T_TH
																																																																																																																																												})),
																																																																																																																																												new HTMLAttributeDef("height",
																																																																																																																																														HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																														HTMLValueDomains.D_LENGTH,
																																																																																																																																														HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																														new Comment("override height"),
																																																																																																																																														new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																HTMLTags.T_IMG, HTMLTags.T_OBJECT
																																																																																																																																														})),
																																																																																																																																														new HTMLAttributeDef("height",
																																																																																																																																																HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																HTMLValueDomains.D_LENGTH,
																																																																																																																																																HTMLAttributeDef.DEFAULT_VALUE_REQUIRED,
																																																																																																																																																new Comment("initial height"),
																																																																																																																																																new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																		HTMLTags.T_APPLET
																																																																																																																																																})),

																																																																																																																																																//dedi sa
																																																																																																																																																new HTMLAttributeDef("href",
																																																																																																																																																		HTMLAttributeDef.RENDERING_EFFECT_BY_PRESENCE,
																																																																																																																																																		HTMLValueDomains.D_URI,
																																																																																																																																																		HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																		new Comment("URI for linked resource"),
																																																																																																																																																		new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																				HTMLTags.T_A, HTMLTags.T_AREA, HTMLTags.T_LINK
																																																																																																																																																		})),
																																																																																																																																																		//dedi sa
																																																																																																																																																		new HTMLAttributeDef("href",
																																																																																																																																																				HTMLAttributeDef.RENDERING_EFFECT_BY_PRESENCE,
																																																																																																																																																				HTMLValueDomains.D_URI,
																																																																																																																																																				HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																				new Comment("URI that acts as base URI"),
																																																																																																																																																				new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																						HTMLTags.T_BASE
																																																																																																																																																				})),
																																																																																																																																																				//swing special
																																																																																																																																																				new HTMLAttributeDef("href",
																																																																																																																																																						HTMLAttributeDef.RENDERING_EFFECT_BY_PRESENCE,
																																																																																																																																																						HTMLValueDomains.D_URI,
																																																																																																																																																						HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																						new Comment("URI that acts as base URI"),
																																																																																																																																																						new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																								HTMLTags.T_CONTENT, HTMLTags.T_IMG
																																																																																																																																																						})),

																																																																																																																																																						new HTMLAttributeDef("hreflang",
																																																																																																																																																								HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																								HTMLValueDomains.D_LANGUAGECODE,
																																																																																																																																																								HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																								new Comment("language code"),
																																																																																																																																																								new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																										HTMLTags.T_A, HTMLTags.T_LINK
																																																																																																																																																								})),

																																																																																																																																																								new HTMLAttributeDef("hspace",
																																																																																																																																																										HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																										HTMLValueDomains.D_PIXELS,
																																																																																																																																																										HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																										new Comment("horizontal gutter"),
																																																																																																																																																										new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																												HTMLTags.T_APPLET, HTMLTags.T_IMG, HTMLTags.T_OBJECT
																																																																																																																																																										})),

																																																																																																																																																										new HTMLAttributeDef("http-equiv",
																																																																																																																																																												HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																												HTMLValueDomains.D_NAME,
																																																																																																																																																												HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																												new Comment("HTTP response header name"),
																																																																																																																																																												new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																														HTMLTags.T_META
																																																																																																																																																												})),

																																																																																																																																																												new HTMLAttributeDef("id",
																																																																																																																																																														HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																														HTMLValueDomains.D_ID,
																																																																																																																																																														HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																														new Comment("document-wide unique id"),
																																																																																																																																																														new HTMLNegTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																HTMLTags.T_BASE, HTMLTags.T_HEAD, HTMLTags.T_HTML, HTMLTags.T_META,
																																																																																																																																																																HTMLTags.T_SCRIPT, HTMLTags.T_STYLE, HTMLTags.T_TITLE
																																																																																																																																																														})),

																																																																																																																																																														new HTMLAttributeDef("ismap",
																																																																																																																																																																HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																HTMLValueDomains.D_EMPTY,
																																																																																																																																																																HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																new Comment("use server-side image map"),
																																																																																																																																																																new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																		HTMLTags.T_IMG, HTMLTags.T_INPUT
																																																																																																																																																																})),

																																																																																																																																																																new HTMLAttributeDef("label",
																																																																																																																																																																		HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																		HTMLValueDomains.D_TEXT,
																																																																																																																																																																		HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																		new Comment("for use in hierarchical menus"),
																																																																																																																																																																		new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																				HTMLTags.T_OPTION
																																																																																																																																																																		})),
																																																																																																																																																																		new HTMLAttributeDef("label",
																																																																																																																																																																				HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																				HTMLValueDomains.D_TEXT,
																																																																																																																																																																				HTMLAttributeDef.DEFAULT_VALUE_REQUIRED,
																																																																																																																																																																				new Comment("for use in hierarchical menus"),
																																																																																																																																																																				new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																						HTMLTags.T_OPTGROUP
																																																																																																																																																																				})),

																																																																																																																																																																				new HTMLAttributeDef("lang",
																																																																																																																																																																						HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																						HTMLValueDomains.D_LANGUAGECODE,
																																																																																																																																																																						HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																						new Comment("language code"),
																																																																																																																																																																						new HTMLNegTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																								HTMLTags.T_BASE, HTMLTags.T_BASEFONT, HTMLTags.T_BR, HTMLTags.T_FRAME,
																																																																																																																																																																								HTMLTags.T_FRAMESET, HTMLTags.T_IFRAME, HTMLTags.T_PARAM, HTMLTags.T_SCRIPT
																																																																																																																																																																						})),

																																																																																																																																																																						new HTMLAttributeDef("language",
																																																																																																																																																																								HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																								HTMLValueDomains.D_CDATA,
																																																																																																																																																																								HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																								new Comment("predefined script language name"),
																																																																																																																																																																								new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																										HTMLTags.T_SCRIPT
																																																																																																																																																																								})),

																																																																																																																																																																								new HTMLAttributeDef("link",
																																																																																																																																																																										HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																										HTMLValueDomains.D_COLOR,
																																																																																																																																																																										HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																										new Comment("color of links"),
																																																																																																																																																																										new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																												HTMLTags.T_BODY
																																																																																																																																																																										})),

																																																																																																																																																																										new HTMLAttributeDef("longdesc",
																																																																																																																																																																												HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																												HTMLValueDomains.D_URI,
																																																																																																																																																																												HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																												new Comment("link to long description (complements alt)"),
																																																																																																																																																																												new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																														HTMLTags.T_IMG
																																																																																																																																																																												})),
																																																																																																																																																																												new HTMLAttributeDef("longdesc",
																																																																																																																																																																														HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																														HTMLValueDomains.D_URI,
																																																																																																																																																																														HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																														new Comment("link to long description (complements title)"),
																																																																																																																																																																														new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																HTMLTags.T_FRAME, HTMLTags.T_IFRAME
																																																																																																																																																																														})),

																																																																																																																																																																														new HTMLAttributeDef("marginheight",
																																																																																																																																																																																HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																HTMLValueDomains.D_PIXELS,
																																																																																																																																																																																HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																new Comment("margin height in pixels"),
																																																																																																																																																																																new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																		HTMLTags.T_FRAME, HTMLTags.T_IFRAME,
																																																																																																																																																																																		//html-standard violations
																																																																																																																																																																																		HTMLTags.T_BODY
																																																																																																																																																																																})),

																																																																																																																																																																																new HTMLAttributeDef("marginwidth",
																																																																																																																																																																																		HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																		HTMLValueDomains.D_PIXELS,
																																																																																																																																																																																		HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																		new Comment("margin widths in pixels"),
																																																																																																																																																																																		new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																				HTMLTags.T_FRAME, HTMLTags.T_IFRAME,
																																																																																																																																																																																				//html-standard violations
																																																																																																																																																																																				HTMLTags.T_BODY
																																																																																																																																																																																		})),

																																																																																																																																																																																		new HTMLAttributeDef("maxlength",
																																																																																																																																																																																				HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																				HTMLValueDomains.D_NUMBER,
																																																																																																																																																																																				HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																				new Comment("max chars for text fields"),
																																																																																																																																																																																				new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																						HTMLTags.T_INPUT
																																																																																																																																																																																				})),

																																																																																																																																																																																				new HTMLAttributeDef("media",
																																																																																																																																																																																						HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																						HTMLValueDomains.D_MEDIADESC,
																																																																																																																																																																																						HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																						new Comment("designed for use with these media"),
																																																																																																																																																																																						new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																								HTMLTags.T_STYLE
																																																																																																																																																																																						})),
																																																																																																																																																																																						new HTMLAttributeDef("media",
																																																																																																																																																																																								HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																								HTMLValueDomains.D_MEDIADESC,
																																																																																																																																																																																								HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																								new Comment("for rendering on these media"),
																																																																																																																																																																																								new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																										HTMLTags.T_LINK
																																																																																																																																																																																								})),

																																																																																																																																																																																								new HTMLAttributeDef("method",
																																																																																																																																																																																										HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																										new HTMLValueDomain(lxEnumValueDef.class,
																																																																																																																																																																																												ListUtils.toSet(new String[] {
																																																																																																																																																																																														"GET", "POST"
																																																																																																																																																																																												}),
																																																																																																																																																																																												null),
																																																																																																																																																																																												"GET",
																																																																																																																																																																																												new Comment("HTTP method used to submit the form"),
																																																																																																																																																																																												new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																														HTMLTags.T_FORM
																																																																																																																																																																																												})),

																																																																																																																																																																																												//nededi sa
																																																																																																																																																																																												new HTMLAttributeDef("multiple",
																																																																																																																																																																																														HTMLAttributeDef.RENDERING_EFFECT_BY_PRESENCE,
																																																																																																																																																																																														HTMLValueDomains.D_EMPTY,
																																																																																																																																																																																														HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																														new Comment("default is single selection"),
																																																																																																																																																																																														new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																HTMLTags.T_SELECT
																																																																																																																																																																																														})),

																																																																																																																																																																																														new HTMLAttributeDef("name",
																																																																																																																																																																																																HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																HTMLValueDomains.D_CDATA,
																																																																																																																																																																																																HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																new Comment(""),
																																																																																																																																																																																																new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																		HTMLTags.T_BUTTON, HTMLTags.T_TEXTAREA
																																																																																																																																																																																																})),
																																																																																																																																																																																																new HTMLAttributeDef("name",
																																																																																																																																																																																																		HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																		HTMLValueDomains.D_CDATA,
																																																																																																																																																																																																		HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																		new Comment("allows applets to find each other"),
																																																																																																																																																																																																		new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																				HTMLTags.T_APPLET
																																																																																																																																																																																																		})),
																																																																																																																																																																																																		new HTMLAttributeDef("name",
																																																																																																																																																																																																				HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																				HTMLValueDomains.D_CDATA,
																																																																																																																																																																																																				HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																				new Comment("field name"),
																																																																																																																																																																																																				new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																						HTMLTags.T_SELECT
																																																																																																																																																																																																				})),
																																																																																																																																																																																																				new HTMLAttributeDef("name",
																																																																																																																																																																																																						HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																						HTMLValueDomains.D_CDATA,
																																																																																																																																																																																																						HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																						new Comment("name of form for scripting"),
																																																																																																																																																																																																						new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																								HTMLTags.T_FORM
																																																																																																																																																																																																						})),
																																																																																																																																																																																																						new HTMLAttributeDef("name",
																																																																																																																																																																																																								HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																								HTMLValueDomains.D_CDATA,
																																																																																																																																																																																																								HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																								new Comment("name of frame for targetting"),
																																																																																																																																																																																																								new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																										HTMLTags.T_FRAME, HTMLTags.T_IFRAME
																																																																																																																																																																																																								})),
																																																																																																																																																																																																								new HTMLAttributeDef("name",
																																																																																																																																																																																																										HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																										HTMLValueDomains.D_CDATA,
																																																																																																																																																																																																										HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																										new Comment("name of image for scripting"),
																																																																																																																																																																																																										new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																												HTMLTags.T_IMG
																																																																																																																																																																																																										})),
																																																																																																																																																																																																										new HTMLAttributeDef("name",
																																																																																																																																																																																																												HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																												HTMLValueDomains.D_CDATA,
																																																																																																																																																																																																												HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																												new Comment("named link end"),
																																																																																																																																																																																																												new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																														HTMLTags.T_A
																																																																																																																																																																																																												})),
																																																																																																																																																																																																												new HTMLAttributeDef("name",
																																																																																																																																																																																																														HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																														HTMLValueDomains.D_CDATA,
																																																																																																																																																																																																														HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																														new Comment("submit as part of form"),
																																																																																																																																																																																																														new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																HTMLTags.T_INPUT, HTMLTags.T_OBJECT
																																																																																																																																																																																																														})),
																																																																																																																																																																																																														new HTMLAttributeDef("name",
																																																																																																																																																																																																																HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																HTMLValueDomains.D_CDATA,
																																																																																																																																																																																																																HTMLAttributeDef.DEFAULT_VALUE_REQUIRED,
																																																																																																																																																																																																																new Comment("for reference by usemap"),
																																																																																																																																																																																																																new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																		HTMLTags.T_MAP
																																																																																																																																																																																																																})),

																																																																																																																																																																																																																new HTMLAttributeDef("name",
																																																																																																																																																																																																																		HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																		HTMLValueDomains.D_CDATA,
																																																																																																																																																																																																																		HTMLAttributeDef.DEFAULT_VALUE_REQUIRED,
																																																																																																																																																																																																																		new Comment("property name"),
																																																																																																																																																																																																																		new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																				HTMLTags.T_PARAM
																																																																																																																																																																																																																		})),
																																																																																																																																																																																																																		new HTMLAttributeDef("name",
																																																																																																																																																																																																																				HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																				HTMLValueDomains.D_NAME,
																																																																																																																																																																																																																				HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																				new Comment("metainformation name"),
																																																																																																																																																																																																																				new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																						HTMLTags.T_META
																																																																																																																																																																																																																				})),

																																																																																																																																																																																																																				new HTMLAttributeDef("nohref",
																																																																																																																																																																																																																						HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																						HTMLValueDomains.D_EMPTY,
																																																																																																																																																																																																																						HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																						new Comment("this region has no action"),
																																																																																																																																																																																																																						new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																								HTMLTags.T_AREA
																																																																																																																																																																																																																						})),

																																																																																																																																																																																																																						new HTMLAttributeDef("noresize",
																																																																																																																																																																																																																								HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																								HTMLValueDomains.D_EMPTY,
																																																																																																																																																																																																																								HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																								new Comment("allow users to resize frames?"),
																																																																																																																																																																																																																								new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																										HTMLTags.T_FRAME
																																																																																																																																																																																																																								})),

																																																																																																																																																																																																																								//nededi sa
																																																																																																																																																																																																																								new HTMLAttributeDef("noshade",
																																																																																																																																																																																																																										HTMLAttributeDef.RENDERING_EFFECT_BY_PRESENCE,
																																																																																																																																																																																																																										HTMLValueDomains.D_EMPTY,
																																																																																																																																																																																																																										HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																										new Comment(""),
																																																																																																																																																																																																																										new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																												HTMLTags.T_HR
																																																																																																																																																																																																																										})),

																																																																																																																																																																																																																										new HTMLAttributeDef("nowrap",
																																																																																																																																																																																																																												HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																												HTMLValueDomains.D_EMPTY,
																																																																																																																																																																																																																												HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																												new Comment("suppress word wrap"),
																																																																																																																																																																																																																												new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																														HTMLTags.T_TD, HTMLTags.T_TH,
																																																																																																																																																																																																																														//swing special
																																																																																																																																																																																																																														HTMLTags.T_TR
																																																																																																																																																																																																																												})),

																																																																																																																																																																																																																												new HTMLAttributeDef("object",
																																																																																																																																																																																																																														HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																														HTMLValueDomains.D_CDATA,
																																																																																																																																																																																																																														HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																														new Comment("serialized applet file"),
																																																																																																																																																																																																																														new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																HTMLTags.T_APPLET
																																																																																																																																																																																																																														})),

																																																																																																																																																																																																																														new HTMLAttributeDef("onblur",
																																																																																																																																																																																																																																HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																HTMLValueDomains.D_SCRIPT,
																																																																																																																																																																																																																																HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																new Comment("the element lost the focus"),
																																																																																																																																																																																																																																new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																		HTMLTags.T_A, HTMLTags.T_AREA, HTMLTags.T_BUTTON, HTMLTags.T_INPUT, HTMLTags.T_LABEL, HTMLTags.T_SELECT, HTMLTags.T_TEXTAREA
																																																																																																																																																																																																																																})),

																																																																																																																																																																																																																																new HTMLAttributeDef("onchange",
																																																																																																																																																																																																																																		HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																		HTMLValueDomains.D_SCRIPT,
																																																																																																																																																																																																																																		HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																		new Comment("the element value was changed"),
																																																																																																																																																																																																																																		new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																				HTMLTags.T_INPUT, HTMLTags.T_SELECT, HTMLTags.T_TEXTAREA
																																																																																																																																																																																																																																		})),

																																																																																																																																																																																																																																		new HTMLAttributeDef("onclick",
																																																																																																																																																																																																																																				HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																				HTMLValueDomains.D_SCRIPT,
																																																																																																																																																																																																																																				HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																				new Comment("a pointer button was clicked"),
																																																																																																																																																																																																																																				new HTMLNegTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																						HTMLTags.T_APPLET, HTMLTags.T_BASE, HTMLTags.T_BASEFONT, HTMLTags.T_BDO,
																																																																																																																																																																																																																																						HTMLTags.T_BR, HTMLTags.T_FONT, HTMLTags.T_FRAME, HTMLTags.T_FRAMESET,
																																																																																																																																																																																																																																						HTMLTags.T_HEAD, HTMLTags.T_HTML, HTMLTags.T_IFRAME, HTMLTags.T_ISINDEX,
																																																																																																																																																																																																																																						HTMLTags.T_META, HTMLTags.T_PARAM, HTMLTags.T_SCRIPT, HTMLTags.T_STYLE,
																																																																																																																																																																																																																																						HTMLTags.T_TITLE
																																																																																																																																																																																																																																				})),

																																																																																																																																																																																																																																				new HTMLAttributeDef("ondblclick",
																																																																																																																																																																																																																																						HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																						HTMLValueDomains.D_SCRIPT,
																																																																																																																																																																																																																																						HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																						new Comment("a pointer button was double clicked"),
																																																																																																																																																																																																																																						new HTMLNegTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																								HTMLTags.T_APPLET, HTMLTags.T_BASE, HTMLTags.T_BASEFONT, HTMLTags.T_BDO,
																																																																																																																																																																																																																																								HTMLTags.T_BR, HTMLTags.T_FONT, HTMLTags.T_FRAME, HTMLTags.T_FRAMESET,
																																																																																																																																																																																																																																								HTMLTags.T_HEAD, HTMLTags.T_HTML, HTMLTags.T_IFRAME, HTMLTags.T_ISINDEX,
																																																																																																																																																																																																																																								HTMLTags.T_META, HTMLTags.T_PARAM, HTMLTags.T_SCRIPT, HTMLTags.T_STYLE,
																																																																																																																																																																																																																																								HTMLTags.T_TITLE
																																																																																																																																																																																																																																						})),

																																																																																																																																																																																																																																						new HTMLAttributeDef("onfocus",
																																																																																																																																																																																																																																								HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																								HTMLValueDomains.D_SCRIPT,
																																																																																																																																																																																																																																								HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																								new Comment("the element got the focus"),
																																																																																																																																																																																																																																								new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																										HTMLTags.T_A, HTMLTags.T_AREA, HTMLTags.T_BUTTON, HTMLTags.T_INPUT, HTMLTags.T_LABEL, HTMLTags.T_SELECT, HTMLTags.T_TEXTAREA
																																																																																																																																																																																																																																								})),

																																																																																																																																																																																																																																								new HTMLAttributeDef("onkeydown",
																																																																																																																																																																																																																																										HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																										HTMLValueDomains.D_SCRIPT,
																																																																																																																																																																																																																																										HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																										new Comment("a key was pressed down"),
																																																																																																																																																																																																																																										new HTMLNegTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																												HTMLTags.T_APPLET, HTMLTags.T_BASE, HTMLTags.T_BASEFONT, HTMLTags.T_BDO,
																																																																																																																																																																																																																																												HTMLTags.T_BR, HTMLTags.T_FONT, HTMLTags.T_FRAME, HTMLTags.T_FRAMESET,
																																																																																																																																																																																																																																												HTMLTags.T_HEAD, HTMLTags.T_HTML, HTMLTags.T_IFRAME, HTMLTags.T_ISINDEX,
																																																																																																																																																																																																																																												HTMLTags.T_META, HTMLTags.T_PARAM, HTMLTags.T_SCRIPT, HTMLTags.T_STYLE,
																																																																																																																																																																																																																																												HTMLTags.T_TITLE
																																																																																																																																																																																																																																										})),

																																																																																																																																																																																																																																										new HTMLAttributeDef("onkeypress",
																																																																																																																																																																																																																																												HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																												HTMLValueDomains.D_SCRIPT,
																																																																																																																																																																																																																																												HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																												new Comment("a key was pressed and released"),
																																																																																																																																																																																																																																												new HTMLNegTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																														HTMLTags.T_APPLET, HTMLTags.T_BASE, HTMLTags.T_BASEFONT, HTMLTags.T_BDO,
																																																																																																																																																																																																																																														HTMLTags.T_BR, HTMLTags.T_FONT, HTMLTags.T_FRAME, HTMLTags.T_FRAMESET,
																																																																																																																																																																																																																																														HTMLTags.T_HEAD, HTMLTags.T_HTML, HTMLTags.T_IFRAME, HTMLTags.T_ISINDEX,
																																																																																																																																																																																																																																														HTMLTags.T_META, HTMLTags.T_PARAM, HTMLTags.T_SCRIPT, HTMLTags.T_STYLE,
																																																																																																																																																																																																																																														HTMLTags.T_TITLE
																																																																																																																																																																																																																																												})),

																																																																																																																																																																																																																																												new HTMLAttributeDef("onkeyup",
																																																																																																																																																																																																																																														HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																														HTMLValueDomains.D_SCRIPT,
																																																																																																																																																																																																																																														HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																														new Comment("a key was released"),
																																																																																																																																																																																																																																														new HTMLNegTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																HTMLTags.T_APPLET, HTMLTags.T_BASE, HTMLTags.T_BASEFONT, HTMLTags.T_BDO,
																																																																																																																																																																																																																																																HTMLTags.T_BR, HTMLTags.T_FONT, HTMLTags.T_FRAME, HTMLTags.T_FRAMESET,
																																																																																																																																																																																																																																																HTMLTags.T_HEAD, HTMLTags.T_HTML, HTMLTags.T_IFRAME, HTMLTags.T_ISINDEX,
																																																																																																																																																																																																																																																HTMLTags.T_META, HTMLTags.T_PARAM, HTMLTags.T_SCRIPT, HTMLTags.T_STYLE,
																																																																																																																																																																																																																																																HTMLTags.T_TITLE
																																																																																																																																																																																																																																														})),

																																																																																																																																																																																																																																														new HTMLAttributeDef("onload",
																																																																																																																																																																																																																																																HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																HTMLValueDomains.D_SCRIPT,
																																																																																																																																																																																																																																																HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																new Comment("all the frames have been loaded"),
																																																																																																																																																																																																																																																new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																		HTMLTags.T_FRAMESET
																																																																																																																																																																																																																																																})),
																																																																																																																																																																																																																																																new HTMLAttributeDef("onload",
																																																																																																																																																																																																																																																		HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																		HTMLValueDomains.D_SCRIPT,
																																																																																																																																																																																																																																																		HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																		new Comment("the document has been loaded"),
																																																																																																																																																																																																																																																		new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																				HTMLTags.T_BODY
																																																																																																																																																																																																																																																		})),

																																																																																																																																																																																																																																																		new HTMLAttributeDef("onmousedown",
																																																																																																																																																																																																																																																				HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																				HTMLValueDomains.D_SCRIPT,
																																																																																																																																																																																																																																																				HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																				new Comment("a pointer button was pressed down"),
																																																																																																																																																																																																																																																				new HTMLNegTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																						HTMLTags.T_APPLET, HTMLTags.T_BASE, HTMLTags.T_BASEFONT, HTMLTags.T_BDO,
																																																																																																																																																																																																																																																						HTMLTags.T_BR, HTMLTags.T_FONT, HTMLTags.T_FRAME, HTMLTags.T_FRAMESET,
																																																																																																																																																																																																																																																						HTMLTags.T_HEAD, HTMLTags.T_HTML, HTMLTags.T_IFRAME, HTMLTags.T_ISINDEX,
																																																																																																																																																																																																																																																						HTMLTags.T_META, HTMLTags.T_PARAM, HTMLTags.T_SCRIPT, HTMLTags.T_STYLE,
																																																																																																																																																																																																																																																						HTMLTags.T_TITLE
																																																																																																																																																																																																																																																				})),

																																																																																																																																																																																																																																																				new HTMLAttributeDef("onmousemove",
																																																																																																																																																																																																																																																						HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																						HTMLValueDomains.D_SCRIPT,
																																																																																																																																																																																																																																																						HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																						new Comment("a pointer was moved within"),
																																																																																																																																																																																																																																																						new HTMLNegTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																								HTMLTags.T_APPLET, HTMLTags.T_BASE, HTMLTags.T_BASEFONT, HTMLTags.T_BDO,
																																																																																																																																																																																																																																																								HTMLTags.T_BR, HTMLTags.T_FONT, HTMLTags.T_FRAME, HTMLTags.T_FRAMESET,
																																																																																																																																																																																																																																																								HTMLTags.T_HEAD, HTMLTags.T_HTML, HTMLTags.T_IFRAME, HTMLTags.T_ISINDEX,
																																																																																																																																																																																																																																																								HTMLTags.T_META, HTMLTags.T_PARAM, HTMLTags.T_SCRIPT, HTMLTags.T_STYLE,
																																																																																																																																																																																																																																																								HTMLTags.T_TITLE
																																																																																																																																																																																																																																																						})),

																																																																																																																																																																																																																																																						new HTMLAttributeDef("onmouseout",
																																																																																																																																																																																																																																																								HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																								HTMLValueDomains.D_SCRIPT,
																																																																																																																																																																																																																																																								HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																								new Comment("a pointer was moved away"),
																																																																																																																																																																																																																																																								new HTMLNegTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																										HTMLTags.T_APPLET, HTMLTags.T_BASE, HTMLTags.T_BASEFONT, HTMLTags.T_BDO,
																																																																																																																																																																																																																																																										HTMLTags.T_BR, HTMLTags.T_FONT, HTMLTags.T_FRAME, HTMLTags.T_FRAMESET,
																																																																																																																																																																																																																																																										HTMLTags.T_HEAD, HTMLTags.T_HTML, HTMLTags.T_IFRAME, HTMLTags.T_ISINDEX,
																																																																																																																																																																																																																																																										HTMLTags.T_META, HTMLTags.T_PARAM, HTMLTags.T_SCRIPT, HTMLTags.T_STYLE,
																																																																																																																																																																																																																																																										HTMLTags.T_TITLE
																																																																																																																																																																																																																																																								})),

																																																																																																																																																																																																																																																								new HTMLAttributeDef("onmouseover",
																																																																																																																																																																																																																																																										HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																										HTMLValueDomains.D_SCRIPT,
																																																																																																																																																																																																																																																										HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																										new Comment("a pointer was moved onto"),
																																																																																																																																																																																																																																																										new HTMLNegTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																												HTMLTags.T_APPLET, HTMLTags.T_BASE, HTMLTags.T_BASEFONT, HTMLTags.T_BDO,
																																																																																																																																																																																																																																																												HTMLTags.T_BR, HTMLTags.T_FONT, HTMLTags.T_FRAME, HTMLTags.T_FRAMESET,
																																																																																																																																																																																																																																																												HTMLTags.T_HEAD, HTMLTags.T_HTML, HTMLTags.T_IFRAME, HTMLTags.T_ISINDEX,
																																																																																																																																																																																																																																																												HTMLTags.T_META, HTMLTags.T_PARAM, HTMLTags.T_SCRIPT, HTMLTags.T_STYLE,
																																																																																																																																																																																																																																																												HTMLTags.T_TITLE
																																																																																																																																																																																																																																																										})),

																																																																																																																																																																																																																																																										new HTMLAttributeDef("onmouseup",
																																																																																																																																																																																																																																																												HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																												HTMLValueDomains.D_SCRIPT,
																																																																																																																																																																																																																																																												HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																												new Comment("a pointer button was released"),
																																																																																																																																																																																																																																																												new HTMLNegTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																														HTMLTags.T_APPLET, HTMLTags.T_BASE, HTMLTags.T_BASEFONT, HTMLTags.T_BDO,
																																																																																																																																																																																																																																																														HTMLTags.T_BR, HTMLTags.T_FONT, HTMLTags.T_FRAME, HTMLTags.T_FRAMESET,
																																																																																																																																																																																																																																																														HTMLTags.T_HEAD, HTMLTags.T_HTML, HTMLTags.T_IFRAME, HTMLTags.T_ISINDEX,
																																																																																																																																																																																																																																																														HTMLTags.T_META, HTMLTags.T_PARAM, HTMLTags.T_SCRIPT, HTMLTags.T_STYLE,
																																																																																																																																																																																																																																																														HTMLTags.T_TITLE
																																																																																																																																																																																																																																																												})),

																																																																																																																																																																																																																																																												new HTMLAttributeDef("onreset",
																																																																																																																																																																																																																																																														HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																														HTMLValueDomains.D_SCRIPT,
																																																																																																																																																																																																																																																														HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																														new Comment("the form was reset"),
																																																																																																																																																																																																																																																														new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																HTMLTags.T_FORM
																																																																																																																																																																																																																																																														})),

																																																																																																																																																																																																																																																														new HTMLAttributeDef("onselect",
																																																																																																																																																																																																																																																																HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																HTMLValueDomains.D_SCRIPT,
																																																																																																																																																																																																																																																																HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																new Comment("some text was selected"),
																																																																																																																																																																																																																																																																new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																		HTMLTags.T_INPUT, HTMLTags.T_TEXTAREA
																																																																																																																																																																																																																																																																})),

																																																																																																																																																																																																																																																																new HTMLAttributeDef("onsubmit",
																																																																																																																																																																																																																																																																		HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																		HTMLValueDomains.D_SCRIPT,
																																																																																																																																																																																																																																																																		HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																		new Comment("the form was submitted"),
																																																																																																																																																																																																																																																																		new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																				HTMLTags.T_FORM
																																																																																																																																																																																																																																																																		})),

																																																																																																																																																																																																																																																																		new HTMLAttributeDef("onunload",
																																																																																																																																																																																																																																																																				HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																				HTMLValueDomains.D_SCRIPT,
																																																																																																																																																																																																																																																																				HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																				new Comment("all the frames have been removed"),
																																																																																																																																																																																																																																																																				new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																						HTMLTags.T_FRAMESET
																																																																																																																																																																																																																																																																				})),
																																																																																																																																																																																																																																																																				new HTMLAttributeDef("onunload",
																																																																																																																																																																																																																																																																						HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																						HTMLValueDomains.D_SCRIPT,
																																																																																																																																																																																																																																																																						HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																						new Comment("the document has been removed"),
																																																																																																																																																																																																																																																																						new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																								HTMLTags.T_BODY
																																																																																																																																																																																																																																																																						})),

																																																																																																																																																																																																																																																																						new HTMLAttributeDef("profile",
																																																																																																																																																																																																																																																																								HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																								HTMLValueDomains.D_URI,
																																																																																																																																																																																																																																																																								HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																								new Comment("named dictionary of meta info"),
																																																																																																																																																																																																																																																																								new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																										HTMLTags.T_HEAD
																																																																																																																																																																																																																																																																								})),

																																																																																																																																																																																																																																																																								new HTMLAttributeDef("prompt",
																																																																																																																																																																																																																																																																										HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																										HTMLValueDomains.D_TEXT,
																																																																																																																																																																																																																																																																										HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																										new Comment("prompt message"),
																																																																																																																																																																																																																																																																										new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																												HTMLTags.T_ISINDEX
																																																																																																																																																																																																																																																																										})),

																																																																																																																																																																																																																																																																										//nededi sa
																																																																																																																																																																																																																																																																										new HTMLAttributeDef("readonly",
																																																																																																																																																																																																																																																																												HTMLAttributeDef.RENDERING_EFFECT_BY_PRESENCE,
																																																																																																																																																																																																																																																																												HTMLValueDomains.D_EMPTY,
																																																																																																																																																																																																																																																																												HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																												new Comment(""),
																																																																																																																																																																																																																																																																												new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																														HTMLTags.T_TEXTAREA
																																																																																																																																																																																																																																																																												})),
																																																																																																																																																																																																																																																																												//nededi sa
																																																																																																																																																																																																																																																																												new HTMLAttributeDef("readonly",
																																																																																																																																																																																																																																																																														HTMLAttributeDef.RENDERING_EFFECT_BY_PRESENCE,
																																																																																																																																																																																																																																																																														HTMLValueDomains.D_EMPTY,
																																																																																																																																																																																																																																																																														HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																														new Comment("for text and passwd"),
																																																																																																																																																																																																																																																																														new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																HTMLTags.T_INPUT
																																																																																																																																																																																																																																																																														})),
																																																																																																																																																																																																																																																																														new HTMLAttributeDef("readonly",
																																																																																																																																																																																																																																																																																HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																HTMLValueDomains.D_LINKTYPES,
																																																																																																																																																																																																																																																																																HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																new Comment("forward link types"),
																																																																																																																																																																																																																																																																																new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																		HTMLTags.T_A, HTMLTags.T_LINK
																																																																																																																																																																																																																																																																																})),

																																																																																																																																																																																																																																																																																new HTMLAttributeDef("rev",
																																																																																																																																																																																																																																																																																		HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																		HTMLValueDomains.D_LINKTYPES,
																																																																																																																																																																																																																																																																																		HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																		new Comment("reverse link types"),
																																																																																																																																																																																																																																																																																		new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																				HTMLTags.T_A, HTMLTags.T_LINK
																																																																																																																																																																																																																																																																																		})),

																																																																																																																																																																																																																																																																																		new HTMLAttributeDef("rows",
																																																																																																																																																																																																																																																																																				HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																				HTMLValueDomains.D_MULTILENGTHS,
																																																																																																																																																																																																																																																																																				HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																				new Comment("list of lengths, HTMLTags.T_default: 100% (1 row)"),
																																																																																																																																																																																																																																																																																				new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																						HTMLTags.T_FRAMESET
																																																																																																																																																																																																																																																																																				})),
																																																																																																																																																																																																																																																																																				new HTMLAttributeDef("rows",
																																																																																																																																																																																																																																																																																						HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																						HTMLValueDomains.D_NUMBER,
																																																																																																																																																																																																																																																																																						HTMLAttributeDef.DEFAULT_VALUE_REQUIRED,
																																																																																																																																																																																																																																																																																						new Comment(""),
																																																																																																																																																																																																																																																																																						new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																								HTMLTags.T_TEXTAREA
																																																																																																																																																																																																																																																																																						})),

																																																																																																																																																																																																																																																																																						new HTMLAttributeDef("rowspan",
																																																																																																																																																																																																																																																																																								HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																								HTMLValueDomains.D_NUMBER,
																																																																																																																																																																																																																																																																																								"1",
																																																																																																																																																																																																																																																																																								new Comment("number of rows spanned by cell"),
																																																																																																																																																																																																																																																																																								new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																										HTMLTags.T_TD, HTMLTags.T_TH
																																																																																																																																																																																																																																																																																								})),

																																																																																																																																																																																																																																																																																								new HTMLAttributeDef("rules",
																																																																																																																																																																																																																																																																																										HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																										HTMLValueDomains.D_TRULES,
																																																																																																																																																																																																																																																																																										HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																										new Comment("rulings between rows and cols"),
																																																																																																																																																																																																																																																																																										new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																												HTMLTags.T_TABLE
																																																																																																																																																																																																																																																																																										})),

																																																																																																																																																																																																																																																																																										new HTMLAttributeDef("scheme",
																																																																																																																																																																																																																																																																																												HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																												HTMLValueDomains.D_CDATA,
																																																																																																																																																																																																																																																																																												HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																												new Comment("select form of content"),
																																																																																																																																																																																																																																																																																												new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																														HTMLTags.T_META
																																																																																																																																																																																																																																																																																												})),

																																																																																																																																																																																																																																																																																												new HTMLAttributeDef("scope",
																																																																																																																																																																																																																																																																																														HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																														HTMLValueDomains.D_SCOPE,
																																																																																																																																																																																																																																																																																														HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																														new Comment("scope covered by header cells"),
																																																																																																																																																																																																																																																																																														new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																HTMLTags.T_TD, HTMLTags.T_TH
																																																																																																																																																																																																																																																																																														})),

																																																																																																																																																																																																																																																																																														new HTMLAttributeDef("scrolling",
																																																																																																																																																																																																																																																																																																HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																new HTMLValueDomain(lxEnumValueDef.class,
																																																																																																																																																																																																																																																																																																		ListUtils.toSet(new String[] {
																																																																																																																																																																																																																																																																																																				"yes", "no", "auto"
																																																																																																																																																																																																																																																																																																		}),
																																																																																																																																																																																																																																																																																																		null),
																																																																																																																																																																																																																																																																																																		"auto",
																																																																																																																																																																																																																																																																																																		new Comment("scrollbar or none"),
																																																																																																																																																																																																																																																																																																		new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																				HTMLTags.T_FRAME, HTMLTags.T_IFRAME
																																																																																																																																																																																																																																																																																																		})),

																																																																																																																																																																																																																																																																																																		new HTMLAttributeDef("selected",
																																																																																																																																																																																																																																																																																																				HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																				HTMLValueDomains.D_EMPTY,
																																																																																																																																																																																																																																																																																																				HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																				new Comment(""),
																																																																																																																																																																																																																																																																																																				new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																						HTMLTags.T_OPTION
																																																																																																																																																																																																																																																																																																				})),

																																																																																																																																																																																																																																																																																																				new HTMLAttributeDef("shape",
																																																																																																																																																																																																																																																																																																						HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																						HTMLValueDomains.D_SHAPE,
																																																																																																																																																																																																																																																																																																						"rect",
																																																																																																																																																																																																																																																																																																						new Comment("controls interpretation of coords"),
																																																																																																																																																																																																																																																																																																						new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																								HTMLTags.T_AREA
																																																																																																																																																																																																																																																																																																						})),
																																																																																																																																																																																																																																																																																																						new HTMLAttributeDef("shape",
																																																																																																																																																																																																																																																																																																								HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																								HTMLValueDomains.D_SHAPE,
																																																																																																																																																																																																																																																																																																								"rect",
																																																																																																																																																																																																																																																																																																								new Comment("for use with client-side image maps"),
																																																																																																																																																																																																																																																																																																								new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																										HTMLTags.T_A
																																																																																																																																																																																																																																																																																																								})),

																																																																																																																																																																																																																																																																																																								new HTMLAttributeDef("size",
																																																																																																																																																																																																																																																																																																										HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																										HTMLValueDomains.D_PIXELS,
																																																																																																																																																																																																																																																																																																										HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																										new Comment(""),
																																																																																																																																																																																																																																																																																																										new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																												HTMLTags.T_HR
																																																																																																																																																																																																																																																																																																										})),
																																																																																																																																																																																																																																																																																																										new HTMLAttributeDef("size",
																																																																																																																																																																																																																																																																																																												HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																												HTMLValueDomains.D_CDATA,
																																																																																																																																																																																																																																																																																																												HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																												new Comment("[+|-]nn e.g. size=\"+1\", HTMLTags.T_size=\"4\""),
																																																																																																																																																																																																																																																																																																												new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																														HTMLTags.T_FONT,
																																																																																																																																																																																																																																																																																																														//swing special
																																																																																																																																																																																																																																																																																																														HTMLTags.T_CONTENT, HTMLTags.T_IMG
																																																																																																																																																																																																																																																																																																												})),
																																																																																																																																																																																																																																																																																																												new HTMLAttributeDef("size",
																																																																																																																																																																																																																																																																																																														HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																														HTMLValueDomains.D_CDATA,
																																																																																																																																																																																																																																																																																																														HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																														new Comment("specific to each type of field"),
																																																																																																																																																																																																																																																																																																														new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																HTMLTags.T_INPUT
																																																																																																																																																																																																																																																																																																														})),
																																																																																																																																																																																																																																																																																																														new HTMLAttributeDef("size",
																																																																																																																																																																																																																																																																																																																HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																HTMLValueDomains.D_CDATA,
																																																																																																																																																																																																																																																																																																																HTMLAttributeDef.DEFAULT_VALUE_REQUIRED,
																																																																																																																																																																																																																																																																																																																new Comment("base font size for FONT elements"),
																																																																																																																																																																																																																																																																																																																new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																		HTMLTags.T_BASEFONT
																																																																																																																																																																																																																																																																																																																})),
																																																																																																																																																																																																																																																																																																																new HTMLAttributeDef("size",
																																																																																																																																																																																																																																																																																																																		HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																		HTMLValueDomains.D_NUMBER,
																																																																																																																																																																																																																																																																																																																		HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																		new Comment("rows visible"),
																																																																																																																																																																																																																																																																																																																		new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																				HTMLTags.T_SELECT
																																																																																																																																																																																																																																																																																																																		})),

																																																																																																																																																																																																																																																																																																																		new HTMLAttributeDef("span",
																																																																																																																																																																																																																																																																																																																				HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																				HTMLValueDomains.D_NUMBER,
																																																																																																																																																																																																																																																																																																																				"1",
																																																																																																																																																																																																																																																																																																																				new Comment("COL attributes affect N columns"),
																																																																																																																																																																																																																																																																																																																				new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																						HTMLTags.T_COL
																																																																																																																																																																																																																																																																																																																				})),
																																																																																																																																																																																																																																																																																																																				new HTMLAttributeDef("span",
																																																																																																																																																																																																																																																																																																																						HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																						HTMLValueDomains.D_NUMBER,
																																																																																																																																																																																																																																																																																																																						"1",
																																																																																																																																																																																																																																																																																																																						new Comment("default number of columns in group"),
																																																																																																																																																																																																																																																																																																																						new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																								HTMLTags.T_COLGROUP
																																																																																																																																																																																																																																																																																																																						})),

																																																																																																																																																																																																																																																																																																																						new HTMLAttributeDef("src",
																																																																																																																																																																																																																																																																																																																								HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																								HTMLValueDomains.D_URI,
																																																																																																																																																																																																																																																																																																																								HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																								new Comment("URI for an external script"),
																																																																																																																																																																																																																																																																																																																								new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																										HTMLTags.T_SCRIPT
																																																																																																																																																																																																																																																																																																																								})),
																																																																																																																																																																																																																																																																																																																								new HTMLAttributeDef("src",
																																																																																																																																																																																																																																																																																																																										HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																										HTMLValueDomains.D_URI,
																																																																																																																																																																																																																																																																																																																										HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																										new Comment("for fields with images"),
																																																																																																																																																																																																																																																																																																																										new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																												HTMLTags.T_INPUT
																																																																																																																																																																																																																																																																																																																										})),
																																																																																																																																																																																																																																																																																																																										new HTMLAttributeDef("src",
																																																																																																																																																																																																																																																																																																																												HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																												HTMLValueDomains.D_URI,
																																																																																																																																																																																																																																																																																																																												HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																												new Comment("source of frame content"),
																																																																																																																																																																																																																																																																																																																												new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																														HTMLTags.T_FRAME, HTMLTags.T_IFRAME
																																																																																																																																																																																																																																																																																																																												})),
																																																																																																																																																																																																																																																																																																																												new HTMLAttributeDef("src",
																																																																																																																																																																																																																																																																																																																														HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																														HTMLValueDomains.D_URI,
																																																																																																																																																																																																																																																																																																																														HTMLAttributeDef.DEFAULT_VALUE_REQUIRED,
																																																																																																																																																																																																																																																																																																																														new Comment("URI of image to embed"),
																																																																																																																																																																																																																																																																																																																														new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																HTMLTags.T_IMG
																																																																																																																																																																																																																																																																																																																														})),

																																																																																																																																																																																																																																																																																																																														new HTMLAttributeDef("standby",
																																																																																																																																																																																																																																																																																																																																HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																HTMLValueDomains.D_TEXT,
																																																																																																																																																																																																																																																																																																																																HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																new Comment("message to show while loading"),
																																																																																																																																																																																																																																																																																																																																new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																		HTMLTags.T_OBJECT
																																																																																																																																																																																																																																																																																																																																})),

																																																																																																																																																																																																																																																																																																																																new HTMLAttributeDef("start",
																																																																																																																																																																																																																																																																																																																																		HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																		HTMLValueDomains.D_NUMBER,
																																																																																																																																																																																																																																																																																																																																		HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																		new Comment("starting sequence number"),
																																																																																																																																																																																																																																																																																																																																		new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																				HTMLTags.T_OL
																																																																																																																																																																																																																																																																																																																																		})),

																																																																																																																																																																																																																																																																																																																																		new HTMLAttributeDef("style",
																																																																																																																																																																																																																																																																																																																																				HTMLAttributeDef.RENDERING_EFFECT_BY_EACH_VALUE,
																																																																																																																																																																																																																																																																																																																																				HTMLValueDomains.D_STYLESHEET,
																																																																																																																																																																																																																																																																																																																																				HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																				new Comment("associated style info"),
																																																																																																																																																																																																																																																																																																																																				new HTMLNegTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																						HTMLTags.T_BASE, HTMLTags.T_BASEFONT, HTMLTags.T_HEAD, HTMLTags.T_HTML,
																																																																																																																																																																																																																																																																																																																																						HTMLTags.T_META, HTMLTags.T_PARAM, HTMLTags.T_SCRIPT, HTMLTags.T_STYLE,
																																																																																																																																																																																																																																																																																																																																						HTMLTags.T_TITLE
																																																																																																																																																																																																																																																																																																																																				})),

																																																																																																																																																																																																																																																																																																																																				new HTMLAttributeDef("summary",
																																																																																																																																																																																																																																																																																																																																						HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																						HTMLValueDomains.D_TEXT,
																																																																																																																																																																																																																																																																																																																																						HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																						new Comment("purpose/structure for speech output"),
																																																																																																																																																																																																																																																																																																																																						new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																								HTMLTags.T_TABLE
																																																																																																																																																																																																																																																																																																																																						})),

																																																																																																																																																																																																																																																																																																																																						new HTMLAttributeDef("tabindex",
																																																																																																																																																																																																																																																																																																																																								HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																								HTMLValueDomains.D_NUMBER,
																																																																																																																																																																																																																																																																																																																																								HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																								new Comment("position in tabbing order"),
																																																																																																																																																																																																																																																																																																																																								new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																										HTMLTags.T_A, HTMLTags.T_AREA, HTMLTags.T_BUTTON, HTMLTags.T_INPUT, HTMLTags.T_OBJECT, HTMLTags.T_SELECT, HTMLTags.T_TEXTAREA
																																																																																																																																																																																																																																																																																																																																								})),

																																																																																																																																																																																																																																																																																																																																								new HTMLAttributeDef("target",
																																																																																																																																																																																																																																																																																																																																										HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																										HTMLValueDomains.D_FRAMETARGET,
																																																																																																																																																																																																																																																																																																																																										HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																										new Comment("render in this frame"),
																																																																																																																																																																																																																																																																																																																																										new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																												HTMLTags.T_A, HTMLTags.T_AREA, HTMLTags.T_BASE, HTMLTags.T_FORM, HTMLTags.T_LINK
																																																																																																																																																																																																																																																																																																																																										})),

																																																																																																																																																																																																																																																																																																																																										new HTMLAttributeDef("text",
																																																																																																																																																																																																																																																																																																																																												HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																												HTMLValueDomains.D_COLOR,
																																																																																																																																																																																																																																																																																																																																												HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																												new Comment("document text color"),
																																																																																																																																																																																																																																																																																																																																												new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																														HTMLTags.T_BODY
																																																																																																																																																																																																																																																																																																																																												})),

																																																																																																																																																																																																																																																																																																																																												new HTMLAttributeDef("title",
																																																																																																																																																																																																																																																																																																																																														HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																														HTMLValueDomains.D_TEXT,
																																																																																																																																																																																																																																																																																																																																														HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																														new Comment("advisory title"),
																																																																																																																																																																																																																																																																																																																																														new HTMLNegTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																HTMLTags.T_BASE, HTMLTags.T_BASEFONT, HTMLTags.T_HEAD, HTMLTags.T_HTML,
																																																																																																																																																																																																																																																																																																																																																HTMLTags.T_META, HTMLTags.T_PARAM, HTMLTags.T_SCRIPT, HTMLTags.T_TITLE
																																																																																																																																																																																																																																																																																																																																														})),

																																																																																																																																																																																																																																																																																																																																														new HTMLAttributeDef("type",
																																																																																																																																																																																																																																																																																																																																																HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																HTMLValueDomains.D_CONTENTTYPE,
																																																																																																																																																																																																																																																																																																																																																HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																new Comment("advisory content type"),
																																																																																																																																																																																																																																																																																																																																																new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																		HTMLTags.T_A, HTMLTags.T_LINK
																																																																																																																																																																																																																																																																																																																																																})),
																																																																																																																																																																																																																																																																																																																																																new HTMLAttributeDef("type",
																																																																																																																																																																																																																																																																																																																																																		HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																		HTMLValueDomains.D_CONTENTTYPE,
																																																																																																																																																																																																																																																																																																																																																		HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																		new Comment("content type for data"),
																																																																																																																																																																																																																																																																																																																																																		new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																				HTMLTags.T_OBJECT
																																																																																																																																																																																																																																																																																																																																																		})),
																																																																																																																																																																																																																																																																																																																																																		new HTMLAttributeDef("type",
																																																																																																																																																																																																																																																																																																																																																				HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																				HTMLValueDomains.D_CONTENTTYPE,
																																																																																																																																																																																																																																																																																																																																																				HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																				new Comment("content type for value when valuetype=ref"),
																																																																																																																																																																																																																																																																																																																																																				new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																						HTMLTags.T_PARAM
																																																																																																																																																																																																																																																																																																																																																				})),
																																																																																																																																																																																																																																																																																																																																																				new HTMLAttributeDef("type",
																																																																																																																																																																																																																																																																																																																																																						HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																						HTMLValueDomains.D_CONTENTTYPE,
																																																																																																																																																																																																																																																																																																																																																						HTMLAttributeDef.DEFAULT_VALUE_REQUIRED,
																																																																																																																																																																																																																																																																																																																																																						new Comment("content type of script language"),
																																																																																																																																																																																																																																																																																																																																																						new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																								HTMLTags.T_SCRIPT
																																																																																																																																																																																																																																																																																																																																																						})),
																																																																																																																																																																																																																																																																																																																																																						new HTMLAttributeDef("type",
																																																																																																																																																																																																																																																																																																																																																								HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																								HTMLValueDomains.D_CONTENTTYPE,
																																																																																																																																																																																																																																																																																																																																																								HTMLAttributeDef.DEFAULT_VALUE_REQUIRED,
																																																																																																																																																																																																																																																																																																																																																								new Comment("content type of style language"),
																																																																																																																																																																																																																																																																																																																																																								new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																										HTMLTags.T_STYLE
																																																																																																																																																																																																																																																																																																																																																								})),
																																																																																																																																																																																																																																																																																																																																																								new HTMLAttributeDef("type",
																																																																																																																																																																																																																																																																																																																																																										HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																										HTMLValueDomains.D_INPUTTYPE,
																																																																																																																																																																																																																																																																																																																																																										"TEXT",
																																																																																																																																																																																																																																																																																																																																																										new Comment("what kind of widget is needed"),
																																																																																																																																																																																																																																																																																																																																																										new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																												HTMLTags.T_INPUT
																																																																																																																																																																																																																																																																																																																																																										})),
																																																																																																																																																																																																																																																																																																																																																										new HTMLAttributeDef("type",
																																																																																																																																																																																																																																																																																																																																																												HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																												HTMLValueDomains.D_LISTYLE,
																																																																																																																																																																																																																																																																																																																																																												HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																												new Comment("list item style"),
																																																																																																																																																																																																																																																																																																																																																												new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																														HTMLTags.T_LI
																																																																																																																																																																																																																																																																																																																																																												})),
																																																																																																																																																																																																																																																																																																																																																												new HTMLAttributeDef("type",
																																																																																																																																																																																																																																																																																																																																																														HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																														HTMLValueDomains.D_OLSTYLE,
																																																																																																																																																																																																																																																																																																																																																														HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																														new Comment("numbering style"),
																																																																																																																																																																																																																																																																																																																																																														new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																HTMLTags.T_OL
																																																																																																																																																																																																																																																																																																																																																														})),
																																																																																																																																																																																																																																																																																																																																																														new HTMLAttributeDef("type",
																																																																																																																																																																																																																																																																																																																																																																HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																																HTMLValueDomains.D_ULSTYLE,
																																																																																																																																																																																																																																																																																																																																																																HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																																new Comment("bullet style"),
																																																																																																																																																																																																																																																																																																																																																																new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																		HTMLTags.T_UL
																																																																																																																																																																																																																																																																																																																																																																})),
																																																																																																																																																																																																																																																																																																																																																																new HTMLAttributeDef("type",
																																																																																																																																																																																																																																																																																																																																																																		HTMLAttributeDef.RENDERING_EFFECT_BY_EACH_VALUE,
																																																																																																																																																																																																																																																																																																																																																																		new HTMLValueDomain(lxEnumValueDef.class,
																																																																																																																																																																																																																																																																																																																																																																				ListUtils.toSet(new String[] {
																																																																																																																																																																																																																																																																																																																																																																						"button", "submit", "reset"
																																																																																																																																																																																																																																																																																																																																																																				}),
																																																																																																																																																																																																																																																																																																																																																																				null),
																																																																																																																																																																																																																																																																																																																																																																				"submit",
																																																																																																																																																																																																																																																																																																																																																																				new Comment("for use as form button"),
																																																																																																																																																																																																																																																																																																																																																																				new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																						HTMLTags.T_BUTTON
																																																																																																																																																																																																																																																																																																																																																																				})),

																																																																																																																																																																																																																																																																																																																																																																				new HTMLAttributeDef("usemap",
																																																																																																																																																																																																																																																																																																																																																																						HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																																						HTMLValueDomains.D_URI,
																																																																																																																																																																																																																																																																																																																																																																						HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																																						new Comment("use client-side image map"),
																																																																																																																																																																																																																																																																																																																																																																						new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																								HTMLTags.T_IMG, HTMLTags.T_INPUT, HTMLTags.T_OBJECT
																																																																																																																																																																																																																																																																																																																																																																						})),

																																																																																																																																																																																																																																																																																																																																																																						new HTMLAttributeDef("valign",
																																																																																																																																																																																																																																																																																																																																																																								HTMLAttributeDef.RENDERING_EFFECT_BY_EACH_VALUE,
																																																																																																																																																																																																																																																																																																																																																																								new HTMLValueDomain(lxEnumValueDef.class,
																																																																																																																																																																																																																																																																																																																																																																										ListUtils.toSet(new String[] {
																																																																																																																																																																																																																																																																																																																																																																												"top", "middle", "bottom", "baseline"
																																																																																																																																																																																																																																																																																																																																																																										}),
																																																																																																																																																																																																																																																																																																																																																																										null),
																																																																																																																																																																																																																																																																																																																																																																										HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																																										new Comment("vertical alignment in cells"),
																																																																																																																																																																																																																																																																																																																																																																										new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																												HTMLTags.T_COL, HTMLTags.T_COLGROUP, HTMLTags.T_TBODY, HTMLTags.T_TD, HTMLTags.T_TFOOT, HTMLTags.T_TH, HTMLTags.T_THEAD, HTMLTags.T_TR
																																																																																																																																																																																																																																																																																																																																																																										})),

																																																																																																																																																																																																																																																																																																																																																																										new HTMLAttributeDef("value",
																																																																																																																																																																																																																																																																																																																																																																												HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																																												HTMLValueDomains.D_CDATA,
																																																																																																																																																																																																																																																																																																																																																																												HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																																												new Comment("Specify for radio buttons and checkboxes"),
																																																																																																																																																																																																																																																																																																																																																																												new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																														HTMLTags.T_INPUT
																																																																																																																																																																																																																																																																																																																																																																												})),
																																																																																																																																																																																																																																																																																																																																																																												new HTMLAttributeDef("value",
																																																																																																																																																																																																																																																																																																																																																																														HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																																														HTMLValueDomains.D_CDATA,
																																																																																																																																																																																																																																																																																																																																																																														HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																																														new Comment("defaults to element content"),
																																																																																																																																																																																																																																																																																																																																																																														new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																																HTMLTags.T_OPTION
																																																																																																																																																																																																																																																																																																																																																																														})),
																																																																																																																																																																																																																																																																																																																																																																														new HTMLAttributeDef("value",
																																																																																																																																																																																																																																																																																																																																																																																HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																																																HTMLValueDomains.D_CDATA,
																																																																																																																																																																																																																																																																																																																																																																																HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																																																new Comment("property value"),
																																																																																																																																																																																																																																																																																																																																																																																new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																																		HTMLTags.T_PARAM
																																																																																																																																																																																																																																																																																																																																																																																})),
																																																																																																																																																																																																																																																																																																																																																																																new HTMLAttributeDef("value",
																																																																																																																																																																																																																																																																																																																																																																																		HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																																																		HTMLValueDomains.D_CDATA,
																																																																																																																																																																																																																																																																																																																																																																																		HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																																																		new Comment("sent to server when submitted"),
																																																																																																																																																																																																																																																																																																																																																																																		new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																																				HTMLTags.T_BUTTON
																																																																																																																																																																																																																																																																																																																																																																																		})),
																																																																																																																																																																																																																																																																																																																																																																																		new HTMLAttributeDef("value",
																																																																																																																																																																																																																																																																																																																																																																																				HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																																																				HTMLValueDomains.D_NUMBER,
																																																																																																																																																																																																																																																																																																																																																																																				HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																																																				new Comment("reset sequence number"),
																																																																																																																																																																																																																																																																																																																																																																																				new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																																						HTMLTags.T_LI
																																																																																																																																																																																																																																																																																																																																																																																				})),

																																																																																																																																																																																																																																																																																																																																																																																				new HTMLAttributeDef("valuetype",
																																																																																																																																																																																																																																																																																																																																																																																						HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																																																						new HTMLValueDomain(lxEnumValueDef.class,
																																																																																																																																																																																																																																																																																																																																																																																								ListUtils.toSet(new String[] {
																																																																																																																																																																																																																																																																																																																																																																																										"DATA", "REF", "OBJECT"
																																																																																																																																																																																																																																																																																																																																																																																								}),
																																																																																																																																																																																																																																																																																																																																																																																								null),
																																																																																																																																																																																																																																																																																																																																																																																								"DATA",
																																																																																																																																																																																																																																																																																																																																																																																								new Comment("How to interpret value"),
																																																																																																																																																																																																																																																																																																																																																																																								new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																																										HTMLTags.T_PARAM
																																																																																																																																																																																																																																																																																																																																																																																								})),

																																																																																																																																																																																																																																																																																																																																																																																								new HTMLAttributeDef("version",
																																																																																																																																																																																																																																																																																																																																																																																										HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																																																										HTMLValueDomains.D_CDATA,
																																																																																																																																																																																																																																																																																																																																																																																										"-//W3C//DTD HTML 4.01 Transitional//EN",
																																																																																																																																																																																																																																																																																																																																																																																										new Comment("Constant"),
																																																																																																																																																																																																																																																																																																																																																																																										new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																																												HTMLTags.T_HTML
																																																																																																																																																																																																																																																																																																																																																																																										})),

																																																																																																																																																																																																																																																																																																																																																																																										new HTMLAttributeDef("vlink",
																																																																																																																																																																																																																																																																																																																																																																																												HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																																																												HTMLValueDomains.D_COLOR,
																																																																																																																																																																																																																																																																																																																																																																																												HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																																																												new Comment("color of visited links"),
																																																																																																																																																																																																																																																																																																																																																																																												new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																																														HTMLTags.T_BODY
																																																																																																																																																																																																																																																																																																																																																																																												})),

																																																																																																																																																																																																																																																																																																																																																																																												new HTMLAttributeDef("vspace",
																																																																																																																																																																																																																																																																																																																																																																																														HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																																																														HTMLValueDomains.D_PIXELS,
																																																																																																																																																																																																																																																																																																																																																																																														HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																																																														new Comment("vertical gutter"),
																																																																																																																																																																																																																																																																																																																																																																																														new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																																																HTMLTags.T_APPLET, HTMLTags.T_IMG, HTMLTags.T_OBJECT
																																																																																																																																																																																																																																																																																																																																																																																														})),

																																																																																																																																																																																																																																																																																																																																																																																														new HTMLAttributeDef("width",
																																																																																																																																																																																																																																																																																																																																																																																																HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																																																																HTMLValueDomains.D_LENGTH,
																																																																																																																																																																																																																																																																																																																																																																																																HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																																																																new Comment(""),
																																																																																																																																																																																																																																																																																																																																																																																																new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																																																		HTMLTags.T_HR
																																																																																																																																																																																																																																																																																																																																																																																																})),
																																																																																																																																																																																																																																																																																																																																																																																																new HTMLAttributeDef("width",
																																																																																																																																																																																																																																																																																																																																																																																																		HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																																																																		HTMLValueDomains.D_LENGTH,
																																																																																																																																																																																																																																																																																																																																																																																																		HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																																																																		new Comment("frame width"),
																																																																																																																																																																																																																																																																																																																																																																																																		new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																																																				HTMLTags.T_IFRAME
																																																																																																																																																																																																																																																																																																																																																																																																		})),
																																																																																																																																																																																																																																																																																																																																																																																																		new HTMLAttributeDef("width",
																																																																																																																																																																																																																																																																																																																																																																																																				HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																																																																				HTMLValueDomains.D_LENGTH,
																																																																																																																																																																																																																																																																																																																																																																																																				HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																																																																				new Comment("override width"),
																																																																																																																																																																																																																																																																																																																																																																																																				new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																																																						HTMLTags.T_IMG, HTMLTags.T_OBJECT
																																																																																																																																																																																																																																																																																																																																																																																																				})),
																																																																																																																																																																																																																																																																																																																																																																																																				new HTMLAttributeDef("width",
																																																																																																																																																																																																																																																																																																																																																																																																						HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																																																																						HTMLValueDomains.D_LENGTH,
																																																																																																																																																																																																																																																																																																																																																																																																						HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																																																																						new Comment("table width"),
																																																																																																																																																																																																																																																																																																																																																																																																						new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																																																								HTMLTags.T_TABLE
																																																																																																																																																																																																																																																																																																																																																																																																						})),
																																																																																																																																																																																																																																																																																																																																																																																																						new HTMLAttributeDef("width",
																																																																																																																																																																																																																																																																																																																																																																																																								HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																																																																								HTMLValueDomains.D_LENGTH,
																																																																																																																																																																																																																																																																																																																																																																																																								HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																																																																								new Comment("width for cell"),
																																																																																																																																																																																																																																																																																																																																																																																																								new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																																																										HTMLTags.T_TD, HTMLTags.T_TH
																																																																																																																																																																																																																																																																																																																																																																																																								})),
																																																																																																																																																																																																																																																																																																																																																																																																								new HTMLAttributeDef("width",
																																																																																																																																																																																																																																																																																																																																																																																																										HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																																																																										HTMLValueDomains.D_LENGTH,
																																																																																																																																																																																																																																																																																																																																																																																																										HTMLAttributeDef.DEFAULT_VALUE_REQUIRED,
																																																																																																																																																																																																																																																																																																																																																																																																										new Comment("initial width"),
																																																																																																																																																																																																																																																																																																																																																																																																										new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																																																												HTMLTags.T_APPLET
																																																																																																																																																																																																																																																																																																																																																																																																										})),
																																																																																																																																																																																																																																																																																																																																																																																																										new HTMLAttributeDef("width",
																																																																																																																																																																																																																																																																																																																																																																																																												HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																																																																												HTMLValueDomains.D_MULTILENGTH,
																																																																																																																																																																																																																																																																																																																																																																																																												HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																																																																												new Comment("column width specification"),
																																																																																																																																																																																																																																																																																																																																																																																																												new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																																																														HTMLTags.T_COL
																																																																																																																																																																																																																																																																																																																																																																																																												})),
																																																																																																																																																																																																																																																																																																																																																																																																												new HTMLAttributeDef("width",
																																																																																																																																																																																																																																																																																																																																																																																																														HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																																																																														HTMLValueDomains.D_MULTILENGTH,
																																																																																																																																																																																																																																																																																																																																																																																																														HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																																																																														new Comment("default width for enclosed COLs"),
																																																																																																																																																																																																																																																																																																																																																																																																														new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																																																																HTMLTags.T_COLGROUP
																																																																																																																																																																																																																																																																																																																																																																																																														})),
																																																																																																																																																																																																																																																																																																																																																																																																														new HTMLAttributeDef("width",
																																																																																																																																																																																																																																																																																																																																																																																																																HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																																																																																HTMLValueDomains.D_NUMBER,
																																																																																																																																																																																																																																																																																																																																																																																																																HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																																																																																new Comment(""),
																																																																																																																																																																																																																																																																																																																																																																																																																new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																																																																		HTMLTags.T_PRE
																																																																																																																																																																																																																																																																																																																																																																																																																})),

																																																																																																																																																																																																																																																																																																																																																																																																																//swing special
																																																																																																																																																																																																																																																																																																																																																																																																																new HTMLAttributeDef("a",
																																																																																																																																																																																																																																																																																																																																																																																																																		HTMLAttributeDef.RENDERING_EFFECT_NONE,
																																																																																																																																																																																																																																																																																																																																																																																																																		HTMLValueDomains.D_EMPTY,
																																																																																																																																																																																																																																																																																																																																																																																																																		HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																																																																																		new Comment("swing flatted tag"),
																																																																																																																																																																																																																																																																																																																																																																																																																		new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																																																																				HTMLTags.T_CONTENT, HTMLTags.T_IMG
																																																																																																																																																																																																																																																																																																																																																																																																																		})),
																																																																																																																																																																																																																																																																																																																																																																																																																		new HTMLAttributeDef("b",
																																																																																																																																																																																																																																																																																																																																																																																																																				HTMLAttributeDef.RENDERING_EFFECT_BY_PRESENCE,
																																																																																																																																																																																																																																																																																																																																																																																																																				HTMLValueDomains.D_EMPTY,
																																																																																																																																																																																																																																																																																																																																																																																																																				HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																																																																																				new Comment("swing flatted tag"),
																																																																																																																																																																																																																																																																																																																																																																																																																				new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																																																																						HTMLTags.T_CONTENT, HTMLTags.T_IMG,
																																																																																																																																																																																																																																																																																																																																																																																																																				})),
																																																																																																																																																																																																																																																																																																																																																																																																																				new HTMLAttributeDef("font-weight",
																																																																																																																																																																																																																																																																																																																																																																																																																						HTMLAttributeDef.RENDERING_EFFECT_BY_EACH_VALUE,
																																																																																																																																																																																																																																																																																																																																																																																																																						HTMLValueDomains.D_CDATA,
																																																																																																																																																																																																																																																																																																																																																																																																																						HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																																																																																						new Comment("swing flatted tag"),
																																																																																																																																																																																																																																																																																																																																																																																																																						new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																																																																								HTMLTags.T_CONTENT, HTMLTags.T_IMG,
																																																																																																																																																																																																																																																																																																																																																																																																																						})),
																																																																																																																																																																																																																																																																																																																																																																																																																						new HTMLAttributeDef("font-size",
																																																																																																																																																																																																																																																																																																																																																																																																																								HTMLAttributeDef.RENDERING_EFFECT_BY_EACH_VALUE,
																																																																																																																																																																																																																																																																																																																																																																																																																								HTMLValueDomains.D_PIXELS,
																																																																																																																																																																																																																																																																																																																																																																																																																								HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																																																																																								new Comment("swing flatted tag"),
																																																																																																																																																																																																																																																																																																																																																																																																																								new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																																																																										HTMLTags.T_CONTENT, HTMLTags.T_IMG,
																																																																																																																																																																																																																																																																																																																																																																																																																								})),
																																																																																																																																																																																																																																																																																																																																																																																																																								new HTMLAttributeDef("font-family",
																																																																																																																																																																																																																																																																																																																																																																																																																										HTMLAttributeDef.RENDERING_EFFECT_BY_EACH_VALUE,
																																																																																																																																																																																																																																																																																																																																																																																																																										HTMLValueDomains.D_CDATA,
																																																																																																																																																																																																																																																																																																																																																																																																																										HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																																																																																										new Comment("swing flatted tag"),
																																																																																																																																																																																																																																																																																																																																																																																																																										new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																																																																												HTMLTags.T_CONTENT, HTMLTags.T_IMG,
																																																																																																																																																																																																																																																																																																																																																																																																																										})),
																																																																																																																																																																																																																																																																																																																																																																																																																										new HTMLAttributeDef("font",
																																																																																																																																																																																																																																																																																																																																																																																																																												HTMLAttributeDef.RENDERING_EFFECT_BY_EACH_VALUE,
																																																																																																																																																																																																																																																																																																																																																																																																																												HTMLValueDomains.D_CDATA,
																																																																																																																																																																																																																																																																																																																																																																																																																												HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																																																																																												new Comment("swing flatted tag"),
																																																																																																																																																																																																																																																																																																																																																																																																																												new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																																																																														HTMLTags.T_CONTENT, HTMLTags.T_IMG,
																																																																																																																																																																																																																																																																																																																																																																																																																												})),
																																																																																																																																																																																																																																																																																																																																																																																																																												new HTMLAttributeDef("big",
																																																																																																																																																																																																																																																																																																																																																																																																																														HTMLAttributeDef.RENDERING_EFFECT_BY_PRESENCE,
																																																																																																																																																																																																																																																																																																																																																																																																																														HTMLValueDomains.D_EMPTY,
																																																																																																																																																																																																																																																																																																																																																																																																																														HTMLAttributeDef.DEFAULT_VALUE_IMPLIED,
																																																																																																																																																																																																																																																																																																																																																																																																																														new Comment("swing flatted tag"),
																																																																																																																																																																																																																																																																																																																																																																																																																														new HTMLEnumTagDefSet(new HTMLTagDef[] {
																																																																																																																																																																																																																																																																																																																																																																																																																																HTMLTags.T_CONTENT, HTMLTags.T_IMG,
																																																																																																																																																																																																																																																																																																																																																																																																																														})),

		};


		name2defs = new HashMap<String, List<HTMLAttributeDef>>();
		for (int i=0; i<attdefs.length; i++) {
			HTMLAttributeDef def = attdefs[i];
			List<HTMLAttributeDef> defs = name2defs.get(def.getName());
			if (defs==null) {
				defs = new LinkedList<HTMLAttributeDef>();
				name2defs.put(def.getName(), defs);
			}
			defs.add(def);
		}
	}

	/*
  public static void main(String[] args) {
    HTMLAttributeDef d1 =
      getDefinitionByName("content", "href");
    System.err.println(d1);
    HTMLAttributeDef d2 =
      getDefinitionByName("content2", "href");
    System.err.println(d2);
    HTMLAttributeDef d3 =
      getDefinitionByName("content2", "href2");
    System.err.println(d3);
  }
	 */

} // HTMLAttributes
