package at.tuwien.prip.model.token;

import java.util.ArrayList;
import java.util.List;

import at.tuwien.prip.model.document.semantics.SemanticText;
import at.tuwien.prip.model.document.semantics.WordSemantics;

/**
 * SemanticToken.java
 * 
 *
 *
 * @author mcg <mcgoebel@gmail.com>
 * Sep 10, 2012
 */
public class SemanticToken extends Token {

	private List<WordSemantics> semantics;
	
	private String text;
	
	/**
	 * 
	 * @param text
	 */
	public SemanticToken(String text)
	{
		super(text);
		this.semantics = new ArrayList<WordSemantics>();
	}
	
	/**
	 * 
	 * @param semantic
	 * @param text
	 */
	public SemanticToken(WordSemantics semantic, String text) 
	{
		this(text);
		this.text = text;
		this.semantics.add(semantic);
	}
	
	/**
	 * 
	 * @param text
	 */
	public SemanticToken(SemanticText text)
	{
		this(text.getValue());
		this.text = text.getValue();
		this.semantics = text.getSemantics();
	}
	
	/**
	 * 
	 * @return
	 */
	public boolean containsSemantics (WordSemantics semantic) 
	{
		return semantics.contains(semantic);
	}
	
	/**
	 * 
	 * @return
	 */
	public List<WordSemantics> getSemantics() {
		return semantics;
	}
	
	public String getText() {
		return text;
	}
	
	@Override
	public String toString() {
		return text;
	}
}
